import os
import numpy as np
from PIL import Image
import ktc
import pandas as pd


def softmax(x, axis=1):
    e_x = np.exp(x)
    return e_x / e_x.sum(axis=axis, keepdims=True)


def preprocess(input_file):
    image = Image.open(input_file).convert("RGB")
    image = np.array(image.resize((224,224), Image.BILINEAR)) #/ 255
    image = np.transpose(image, (2, 0, 1))[None]
    return image


nef_file="./models_720.nef"
dir_image = "./images"
result_file = "./nef_docker_results.xlsx"
image_files = [f for f in os.listdir(dir_image) if f.endswith(('jpg'))]
image_files = sorted(image_files)

platform = 720
predictions = []
for idx, image_file in enumerate(image_files):
    print(f'Processing image {idx}/{len(image_files)}: {image_file}')
    input_data = preprocess(os.path.join(dir_image, image_file))
    pred = ktc.kneron_inference([input_data], nef_file = nef_file, input_names=["images"], model_id=32770, platform=platform)
    predictions.append(pred)
predictions = np.vstack(predictions).reshape(len(image_files), -1)
predictions = softmax(predictions)
pred_labels = predictions.argmax(axis=1)
pred_probs = predictions.max(axis=1)
print(f"Nef Labels from docker: {pred_labels} \n")
print(f"Nef Probabilities from docker: {pred_probs} \n\n")

df_final_pred = np.vstack([pred_probs, pred_labels]).T
df_final_pred = pd.DataFrame(df_final_pred, columns=["pred prob", "pred label"])
df = pd.DataFrame(image_files, columns=["image name"])
df_final_pred = df.join(df_final_pred)
df_full_pred = pd.DataFrame(predictions.T, columns=image_files)
with pd.ExcelWriter(result_file) as writer:  
    df_final_pred.to_excel(writer, sheet_name='final pred')
    df_full_pred.to_excel(writer, sheet_name='full pred')