import pandas as pd

# Function to calculate TOP1, TOP2, TOP5, TOP10 differences based on absolute differences
def calculate_top_differences(df1, df2, top_k):
    top_diff = pd.DataFrame()
    for idx in range(1, len(df1.columns)):  # Skip the first column, assuming it's an index or identifier
        col = df1.columns[idx]
        absolute_diff = abs(df1[col] - df2[col])
        sorted_diff = absolute_diff.sort_values(ascending=False).reset_index(drop=True)
        for k in top_k:
            top_diff.at[idx, f'TOP{k}_diff'] = sorted_diff.iloc[k-1]  # Get the k-th largest difference
    return top_diff

docker_result_file = "./nef_docker_results.xlsx"
usb_result_file = "./nef_usb_results.xlsx"
results_compare_file = "./results_compare.xlsx"

# Load predictions
docker_predictions_full = pd.read_excel(docker_result_file, sheet_name='full pred')
docker_predictions_final = pd.read_excel(docker_result_file, sheet_name='final pred')
usb_predictions_full = pd.read_excel(usb_result_file, sheet_name='full pred')
usb_predictions_final = pd.read_excel(usb_result_file, sheet_name='final pred')

# Calculate TOP1, TOP2, TOP5, TOP10 differences for full predictions
top_k = [1, 2, 5, 10]
top_differences_full = calculate_top_differences(docker_predictions_full, usb_predictions_full, top_k)

# Calculate label differences
label_diff = [0] + list(abs(docker_predictions_final.iloc[:, 3] - usb_predictions_final.iloc[:, 3])[1:])
top_differences_full['label_diff'] = label_diff

# Extract image names from the final predictions
image_names = []
image_names[1:] = docker_predictions_final.iloc[:, 1]  # Assuming image names are in the first column

# Insert image names as the first column in the top_differences_full DataFrame
top_differences_full.insert(0, 'image_name', image_names)

# Save results to a new Excel file
with pd.ExcelWriter(results_compare_file) as writer:
    top_differences_full.to_excel(writer, sheet_name='Top_Differences', index=False)
